      subroutine mts_intra_n0(xp0,yp0,zp0,xcm,ycm,zcm,fpx,fpy,fpz
     &     ,ubond_slt,ubond_slv,ubend_slt,ubend_slv,uitors_slt
     &     ,uitors_slv,stressd,tag)

************************************************************************
*     FINTRAP stretching and bending routines for solute.
*     (used only with r-RESPA on)
************************************************************************

      IMPLICIT none

      include 'parst.h'

*----------------------- ARGUMENTS -------------------------------------

      INTEGER tag(*)
      REAL*8  xp0(*),yp0(*),zp0(*),xcm(*),ycm(*),zcm(*),ubend_slt
     &     ,ubond_slt,uitors_slt,ubend_slv,ubond_slv,uitors_slv
     &     ,stressd(3,3)
      REAL*8  fpx(*),fpy(*),fpz(*)

*-------------------- VARIABLES IN COMMONS -----------------------------

      INCLUDE 'cpropar.h'
      REAL*8  fppx(m1,2),fppy(m1,2),fppz(m1,2)
      COMMON /rag1/ fppx,fppy,fppz

*-------------------- LOCAL VARIABLES ----------------------------------
      
      REAL*8  xc,yc,zc,xd,yd,zd,sumx,sumy,sumz,st(3,3)
      INTEGER i,j,i1,count,m
      INCLUDE 'pbc.h'

*=======================================================================
*--- Zero stress tensor ------------------------------------------------
*=======================================================================

      IF(cpress .OR. pressure) THEN
         CALL zero0(stressd,9)
         CALL zero0(st,9)
      END IF

*=======================================================================
*----- Bonded interactions: stretching ------------------------------------
*=======================================================================

      IF(stretch) THEN
         CALL fpbond(ss_index,lstrtch,lstretch,xp0,yp0,zp0,potbo(1,2)
     &        ,potbo(1,1),ubond_slt,ubond_slv,fpx,fpy,fpz)
      END IF

*=======================================================================
*----- Bonded interactions: bending ------------------------------------
*=======================================================================

      IF(bending) THEN
         CALL zeroa(fppx(1,1),fppy(1,1),fppz(1,1),ntap,1)
         CALL zeroa(fppx(1,2),fppy(1,2),fppz(1,2),ntap,1)
         CALL fpbend(ss_index,lbndg,lbend,tag,xp0,yp0,zp0,potbe(1,2)
     &        ,potbe(1,1),potbe(1,4),potbe(1,3),potbe(1,5),potbe(1,6)
     &        ,potbe(1,7),ubend_slt,ubend_slv,fppx,fppy,fppz,m1)

         CALL daxpy(ntap,1.0D0,fppx(1,2),1,fpx,1)
         CALL daxpy(ntap,1.0D0,fppy(1,2),1,fpy,1)
         CALL daxpy(ntap,1.0D0,fppz(1,2),1,fpz,1)
      END IF

*=======================================================================
*----- Bonded interactions: improper torsions --------------------------
*=======================================================================

      CALL fitors(ss_index,litr,litor,xp0,yp0,zp0,potit(1,2),
     x     potit(1,1),potit(1,3),uitors_slt,uitors_slv,fpx,fpy,fpz)

*=======================================================================
*---- Compute stress tensor if coupling is by group --------------------
*=======================================================================

      IF(cpress .OR. pressure .AND. coupl_grp) THEN
         count=0
         DO i=1,nprot
            xc=xcm(i)
            yc=ycm(i)
            zc=zcm(i)
            m=protl(count+1)
            DO i1=1,m
               j=protl(count+1+i1)
               st(1,1)=st(1,1)+xc*fpx(j)
               st(1,2)=st(1,2)+yc*fpx(j)
               st(1,3)=st(1,3)+zc*fpx(j)
               st(2,1)=st(2,1)+xc*fpy(j)
               st(2,2)=st(2,2)+yc*fpy(j)
               st(2,3)=st(2,3)+zc*fpy(j)
               st(3,1)=st(3,1)+xc*fpz(j)
               st(3,2)=st(3,2)+yc*fpz(j)
               st(3,3)=st(3,3)+zc*fpz(j)
            END DO
            count=count+m+1
         END DO
         CALL DGEMM('N','T',3,3,3,1.0D0,co,3,st,3,0.0D0,stressd,3)
      END IF

*=======================================================================
*---- Add contribution to force from bendings among same group ---------
*=======================================================================

      IF(bending) THEN
         CALL daxpy(ntap,1.0D0,fppx(1,1),1,fpx,1)
         CALL daxpy(ntap,1.0D0,fppy(1,1),1,fpy,1)
         CALL daxpy(ntap,1.0D0,fppz(1,1),1,fpz,1)
      END IF

      RETURN
      END
