      PROGRAM main

************************************************************************
*                                                                      *
*                       ORAC of BLAKES 7                               *
*                                                                      *
************************************************************************
*                                                                      *
*                                                                      *
*======================================================================*
*                                                                      *
*                              O R A C                                 *
*                                                                      *
*                           (5.0 Beta Version)                         *
*                                                                      *
*          "A Molecular Dynamics Program to Simulate Complex Systems"  *
*                                                                      *
*              Copyright(C) 1989 - 2008 by M.Marchi and P.Procacci     *
*                                                                      *
*                        All Right Reserved                            *
*                                                                      *
*                                                                      *
*                                                                      *
*                                                                      *
* ORAC is provided "as is" and without any warranty express or implied.*
* The user assumes all risks of using ORAC.                            *
*                                                                      *
* The user may make copies of ORAC for his/her own use, and modify     *
* those copies. The user MAY NOT distribute any copy of the origina    *
* or modified source code to any users at any sites other than his/her *
* own.                                                                 *
*                                                                      *
*                                                                      *
*                                                                      *
*       Written by  Massimo Marchi and Piero Procacci                  *
*                                                                      *
*                                                                      *
*                                                                      *
************************************************************************
*                                                                      *
*                                                                      *
*                                                                      *
************************************************************************


*======================= DECLARATIONS ==================================

      IMPLICIT none

*======================= DECLARATIONS ==================================

      INCLUDE 'parst.h'
      INCLUDE 'cpropar.h'
      INCLUDE 'unit.h'
      INCLUDE 'orac.h'
#ifdef PARALLEL
      include 'crem.inc'
      INCLUDE 'mpif.h'
#endif

*-------------- DEFINITION OF THE SCRATCH COMMON BLOCK -----------------


*-------------------- LOCAL VARIABLES ----------------------------------

      CHARACTER*80 errmsg
      INTEGER i,iret
#ifdef PARALLEL
      integer pnrep
      integer iproc, nproc, ierr, chdir,getcwd
      character (len=512) command,pwd
      character (len=128) directory,file,wdir,file2,array,inputfile
      CHARACTER*130 line
      LOGICAL EXIST
#endif

*==================== EXECUTABLE STATEMENTS ============================

#ifdef PARALLEL
      include 'orac_rem.inc'
#endif 
            
*=======================================================================
*----- Read form input files and initialiase topology arrays -----------
*=======================================================================

      CALL starta(mapnl,mapnl_slv,mapp,mapp_slv)

*=======================================================================
*----- Build the simulation box and, if needed, insert a macromolecule -
*----- in an equilibrated configuration of solvent molecules -----------
*=======================================================================

      CALL bldbox(xp0,yp0,zp0,xpg,ypg,zpg,gh,mapnl,mapnl_slv,iret,errmsg
     &     )

*----------------- If iret .EQ. 1 STOP ** ------------------------------
*                                 ====

      IF(iret .EQ. 2) CALL xerror(errmsg,80,1,20)
      IF(iret.EQ.1) CALL xerror(errmsg,80,1,2)
      IF(.NOT. stoprun) THEN


*=======================================================================
*----- Verify some variables -------------------------------------------
*=======================================================================

         CALL verify_variables(iret,errmsg)
         IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
                    
*=======================================================================
*----- Run an MD simulation or analysis --------------------------------
*=======================================================================

         IF(analys) THEN
            CALL drive_analysis(mapnl,xp0,yp0,zp0,xpg,ypg,zpg,gh,xpcm
     &     ,ypcm,zpcm)
         ELSE IF(mdsim) THEN
         CALL mtsmd(mapnl,xp0,yp0,zp0,xpg,ypg,zpg,gh,xpcm,ypcm,zpcm)
         ELSE IF(minimize .OR. frequencies) THEN
            CALL run_minimize(mapnl,xp0,yp0,zp0,xpg,ypg,zpg,gh,xpcm
     &           ,ypcm,zpcm)
         END IF
      END IF
  
#ifdef PARALLEL
      call MPI_FINALIZE(ierr)
      if(iproc.eq.0) THEN 
        WRITE(6,100)
        WRITE(6,200)
      END IF
#endif
      WRITE(kprint,100)
      WRITE(kprint,200)
      CALL close_files

*================= END OF EXECUTABLE STATEMENTS ========================

100   FORMAT(//
     &     '     *****************************************************',
     &     '*******************'/
     &     '     *                                                    ',
     &     '                  *'/
     &     '     *                        PROGRAM COMPLETED           ',
     &     '                  *'/
     &     '     *                                                    ',
     &     '                  *'/
     &     '     *****************************************************',
     &     '*******************'/)

200   FORMAT(//10x,
     &'|--------------------------------------------------------------|' 
     &,/10x,
     &"|    ``They're unfriendly.  Which is fortunate really;         |"
     &,/10x,
     &"|      they'd be difficult to love.''                          |"
     &,/10x,
     &"|                              . . . Avon    Blake's Seven     |"
     &,/10x,
     &"|                                                              |"
     &,/10x,
     &"|--------------------------------------------------------------|"
     &//)
      STOP
      END

      SUBROUTINE BLSTRIP( STRING, LENGHT )
C  Strip trailing blanks from STRING, returns LENGTH
      PARAMETER( MAXLEN = 80 )
      CHARACTER*80 STRING
      
      LENGHT = MAXLEN+1
1     LENGHT = LENGHT - 1
      IF( LENGHT .EQ. 0 ) RETURN
      IF( STRING(LENGHT:LENGHT) .EQ. ' ' ) GOTO 1

      RETURN
      END
