!     iproc is the process rank
!     para_index is the replica index
!     pnrep is the total number of replicas

!   initialize the MPI environment

      CALL MPI_Init(ierr)
      CALL MPI_Comm_size(MPI_COMM_WORLD, nproc, ierr)
      CALL MPI_Comm_rank(MPI_COMM_WORLD, iproc, ierr)

      file="PAR"
      IF(iproc .EQ. 0) THEN
!   copy the input file to a generic file INPUT
        OPEN (unit=99,file="INPUT")
1000    READ(5,'(a130)',END=2000) line
        WRITE(99,'(a130)') line
        GOTO 1000
2000    CONTINUE
        iret=0
        CLOSE(99)
      END IF

      CALL MPI_Barrier(MPI_COMM_WORLD,ierr) 

! initialize logical variables
      para_restart = .false.
      rem=.false.

! place the current working directory name in wdir
! i = 0 on success
      i=getcwd(wdir)

! directory='cwd/PARXXXX'
      WRITE (directory,'(a,a,a,i4.4)') TRIM(wdir),'/',TRIM(file),iproc
      
      exist=.false.

! check if the INPUT file is present in the PARXXXX directory
      inputfile = TRIM(directory) // '/INPUT'
      INQUIRE(FILE=inputfile,EXIST=exist) 

! try to change the work directory to PARXXXX
      CALL FORT_CSTRING(directory,array)
      i=CHDIR(array)

! consistency check: 
! if there is a PARXXXX directory (i = 0), it must contain 
! an INPUT file from some old run
      if(i.eq.0.neqv.exist) THEN 
        write(6,*) ' I/O Problems with process',iproc,i,exist,inputfile
        STOP
      END IF

! ****************************************************************************************
! ************** if a directory PARXXXX exists, cp the new INPUT file in it **************
! ****************************************************************************************
      if (i.eq.0) then 
        
! the work directory is now PARXXXX
!     copy the new INPUT file 
        
        command = 'bash -c ' // '"' // 'cp -f ../INPUT . >& /dev/null'
     &       // '"'
        
        CALL system (command)
        
      else

! ***********************************************************************************
! ****************** if there is not a directory PARXXXX, create it  ***************
! ***********************************************************************************

! create directory PARXXXX
        command = 'bash -c ' // '"' // 'mkdir' // ' ' // directory //
     &       ' >& /dev/null ' // '"'
        CALL system (command)

! and copy INPUT file in it 
        command = 'bash -c ' // '"' // 'cp '//'INPUT '//directory //
     &       ' >& /dev/null'  // '"'
        
        CALL system (command)
        
! change the current work directory 
        i=CHDIR(array)
        
        if (i.ne.0) then
! error: stop
          WRITE(*,*) "************************************************"
          WRITE(*,*) "*          FATAL ERROR!!!                      *"
          WRITE(*,*) "*  Fatal error in changing directory           *"
          WRITE(*,*) "*  Check that the subdirectories PARXXXX       *"
          WRITE(*,*) "*  were created.                               *"
          WRITE(*,*) "************************************************"
          CALL MPI_Finalize(ierr)
          STOP 
        endif
      endif
      
! all ok: go over and print headings
      if(iproc.eq.0) THEN
        write(*,123) nproc
123     format
     &       (10x,'===================================================='
     &       /10x,'=                                                  ='
     &       /10x,'=    P A R A L L E L    O R A C     S T A R T E D  ='
     &       /10x,'=                                                  ='
     &       /10x,'=    release 5.0 (S. Marsili and P. Procacci)      ='
     &       /10x,'=                                                  ='
     &       /10x,'=            Copyright (c) 2008-2009               ='
     &       /10x,'=                                                  ='
     &       /10x,'=        RUN LAUNCHED ON ',i3
     &       ,' PROCESSORS            ='
     &       /10x,'=                                                  ='
     &       /10x,'===================================================='
     &  )
      end if
      CALL MPI_Barrier(MPI_COMM_WORLD,ierr) 
! refresh wdir

      i=getcwd(wdir)      
