      PROGRAM mbar
! print a two-dimensional PMF from REM simulation data using the MBAR
! method;
! mbar needs a mbar.in input file;
! each row of mbar.in corresponds to a configuration sampled during the
! simulation, without any special order;
! each row contains five numbers:
! 1) temperature index (from 1 to the total number of replica);
! 2) temperature (K);
! 3) potential (kJoule mole -1);
! 4) value of the first collective variable;
! 5) value of the second collective variable;

! mbar prints two output files:
! 1) fes.out is the free energy surface obtained sampling from the
! lowest temperature replica only;
! 2) fes_mbar.out uses the sampling of all the temperatures trough the
! MBAR algorithm;
! both files are input files for the plotmtv program;

! nb!:
! compile only with ifort!!!!!!!!!;
! the collective variables must be two dihedral angles;
! for a large number of points and replicas check the dim.inc file:
! pmax is the maximum number of points;
! rmax is the maximum number of replica;

! todo:
! the plot routine should handle both one and two dimensional FESs and
! general collective variables;
! rewrite in fortran 90/95;
! use SELECTED_REAL_KIND;
! the program should print the final free entropies;



!! lines not needed - GFS 

      IMPLICIT NONE
      INCLUDE 'dim.inc'
      INCLUDE 'const.inc'
      INTEGER iter,itr,p,np,bp,nrep
!!      INTEGER input_unit,output_unit,fes_unit,fes_mbar_unit,ierror
      INTEGER input_unit,fes_unit,fes_mbar_unit,ierror
      INTEGER rp
      INTEGER irep,rep(pmax),n(rmax)
      REAL(16) ep,sum,dpi,nrm,nrmi,it,ie,b0,cv1,cv2
      REAL(16) t(rmax),b(rmax),psi(rmax),dum(rmax),exp_psi(rmax)
      REAL(16) e(pmax),w0(pmax),d(pmax)
      REAL(16) cv(2,pmax)

! initialize some variables
      
      nrep = 0
      DO irep = 1, rmax
        psi(irep) = 0.
        n(irep) = 0.
      ENDDO
      
! open files
      
      input_unit=11
!!      output_unit=12
      fes_unit=13
      fes_mbar_unit=14
      
      OPEN(UNIT = input_unit, FILE = 'mbar.in', STATUS = 'OLD', ACTION =
     &     'READ', IOSTAT = ierror)
      IF(ierror.GT.0) WRITE(0,101)
      
!!      OPEN(UNIT = output_unit, FILE = 'mbar.out', STATUS = 'REPLACE',
!!     &     ACTION='WRITE', IOSTAT = ierror)
!!      IF(ierror.GT.0) WRITE(0,102)
      
      OPEN(UNIT = fes_unit, FILE = 'fes.out', STATUS = 'REPLACE',
     &     ACTION='WRITE', IOSTAT = ierror)
      IF(ierror.GT.0) WRITE(0,107)
      
      OPEN(UNIT = fes_mbar_unit, FILE = 'fes_mbar.out', STATUS =
     &     'REPLACE',ACTION='WRITE', IOSTAT = ierror)
      IF(ierror.GT.0) WRITE(0,108)
      
      
! read input
      
      DO p = 1,pmax
        READ(input_unit,*,IOSTAT = ierror) irep, it, ie, cv1, cv2
        IF(ierror.NE.0) GOTO 1000
        np = p
        rep(p) = irep
        t(irep) = it
        e(p) = ie
        cv(1,p) = cv1
        cv(2,p) = cv2
        IF(nrep.LT.irep) nrep = irep
      ENDDO
      
      WRITE(0,105)
      
1000  IF(ierror.GT.0) THEN
        WRITE(0,103) np + 1
      ELSEIF(ierror.LT.0) THEN
        WRITE(0,104) np
        DO irep = 1,nrep
          n(irep) = 0
        ENDDO
        
        DO p = 1,np
          rp = rep(p)
          n(rp) = n(rp) + 1
        ENDDO

! calculate inverse temperatures        

        DO irep = 1,nrep
          WRITE(0,*) 'Temp ', irep, ' : ', t(irep)
          b(irep) = 1. / (gascon * 0.001 * t(irep))
        ENDDO
        b0 = 1. / (gascon * 0.001 * t0)
        
! iteration (of iterations) starts here 
! calculate the free entropies psi from a subset
! of the samples and use it as an initial guess for a double-size set in
! the next iteration
! for each step new psi are calculated through an iterative procedure
        bp = bigp
        DO iter = 1,niter
          IF(bp.GT.np) GOTO 2000
          WRITE(0,*) 'Using ', bp, ' samples of ', np
          CALL iterative(nrep,bp,rep,psi,n,b,e) 
          bp = bp * 2
          itr = iter
        ENDDO
2000    CONTINUE
        IF(itr.LT.niter) THEN
          WRITE(0,*) 'Using all ', np, ' samples'
          CALL iterative(nrep,np,rep,psi,n,b,e)
        ELSE
          WRITE(0,*) 'Warning: Reached maximum number of iterations'
        ENDIF
! iteration ends here

! calculate the weights w0 at the target temperature from the free entropies
        nrm = 0.
        DO irep = 1,nrep
          exp_psi(irep) = EXP(psi(irep))
        ENDDO
        
        DO p = 1,np         
          sum = 0.
          DO irep = 1,nrep
            sum = sum + exp_psi(irep) * EXP(-b(irep) * e(p)) * n(irep)
          ENDDO
          d(p) = sum
          w0(p) = EXP(- b0 * e(p)) / d(p)
          nrm = nrm + w0(p)
        ENDDO         
        
! normalize weights
        nrmi = 1./nrm
        DO p = 1,np
          w0(p) = w0(p) * nrmi
        ENDDO

! plot a bidimensional PMF
        CALL plotmtv_histog(np,w0,cv,rep,fes_unit ! rimettere a posto
     &       ,fes_mbar_unit) 
        
      ENDIF

      
      CLOSE(input_unit)
!!      CLOSE(output_unit)
      CLOSE(fes_unit)
      CLOSE(fes_mbar_unit)
      
      
101   FORMAT(1X, 'Error opening input file')
102   FORMAT(1X, 'Error opening output file')
103   FORMAT(1X, 'An error occurred reading line', I6)
104   FORMAT(1X, ' Input was OK, there were ', I6, ' lines',/)
105   FORMAT(1X, ' Warning: reached the maximum number of input lines')
106   FORMAT(1X, ' Initial guess from a subset of the structures')
107   FORMAT(1X, 'Error opening fes output file')
108   FORMAT(1X, 'Error opening fes output file')

      
      
      STOP
      END

